@@ -43,15 +43,15 @@ static char ar_c_id[] = "@(#)$Id: ar.c,v 1.12 2010/10/04 21:20:47 cm-msk Exp $";
 #include <signal.h>
 #include <string.h>
 #include <syslog.h>
-#ifdef ARDEBUG
-# include <stdio.h>
-#endif /* ARDEBUG */
+#include <stdio.h>
 
 /* important macros */
 #define AR_MAXHOSTNAMELEN	256
 
 #define	ARDEBUGOUT	"/var/tmp/ardebug.out"
 
+#define	BUFRSZ		1024
+
 #ifndef MAXPACKET
 # define MAXPACKET	8192
 #endif /* ! MAXPACKET */
@@ -1323,8 +1323,6 @@ ar_dispatcher(void *tp)
 			}
 		}
 
-		pthread_mutex_lock(&lib->ar_lock);
-
 		buf = NULL;
 
 		/* read what's available from the nameserver for dispatch */
@@ -1342,13 +1340,16 @@ ar_dispatcher(void *tp)
 				             NULL);
 
 				if (r == (size_t) -1)
+				{
+					pthread_mutex_lock(&lib->ar_lock);
 					continue;
+				}
 
 				buf = lib->ar_querybuf;
 			}
 			else if (lib->ar_tcpmsglen == 0)
 			{
-				u_short len;
+				uint16_t len;
 				_Bool err = FALSE;
 
 				/* get the length */
@@ -1359,9 +1360,14 @@ ar_dispatcher(void *tp)
 				if (r == (size_t) -1)
 				{
 					if (errno == EINTR)
+					{
+						pthread_mutex_lock(&lib->ar_lock);
 						continue;
+					}
 					else
+					{
 						err = TRUE;
+					}
 				}
 				else if (r == 0)
 				{
@@ -1369,11 +1375,14 @@ ar_dispatcher(void *tp)
 				}
 				else if (r < sizeof len)
 				{
+					pthread_mutex_lock(&lib->ar_lock);
 					continue;
 				}
 
 				if (err)
 				{
+					pthread_mutex_lock(&lib->ar_lock);
+
 					/* request a reconnect */
 		     			lib->ar_flags |= AR_FLAG_RECONNECT;
 
@@ -1388,7 +1397,7 @@ ar_dispatcher(void *tp)
 
 				/* allocate a buffer */
 				if (lib->ar_tcpbuf == NULL ||
-				    lib->ar_tcpbuflen < len)
+				    lib->ar_tcpbuflen < ntohs(len))
 				{
 					if (lib->ar_tcpbuf != NULL)
 					{
@@ -1396,8 +1405,9 @@ ar_dispatcher(void *tp)
 						lib->ar_tcpbuf = NULL;
 					}
 
-					lib->ar_tcpbuf = ar_malloc(lib, len);
-					lib->ar_tcpbuflen = len;
+					lib->ar_tcpbuf = ar_malloc(lib,
+					                           ntohs(len));
+					lib->ar_tcpbuflen = ntohs(len);
 				}
 			}
 			else
@@ -1436,6 +1446,8 @@ ar_dispatcher(void *tp)
 
 				if (err)
 				{
+					pthread_mutex_lock(&lib->ar_lock);
+
 					/* request a reconnect */
 		     			lib->ar_flags |= AR_FLAG_RECONNECT;
 
@@ -1453,6 +1465,8 @@ ar_dispatcher(void *tp)
 			}
 		}
 
+		pthread_mutex_lock(&lib->ar_lock);
+
 		if (buf != NULL)		/* something to parse */
 		{
 			_Bool requeued = FALSE;
@@ -2780,3 +2794,122 @@ ar_strerror(int err)
 		return strerror(errno);
 	}
 }
+
+/*
+**  AR_RESOLVCONF -- parse a resolv.conf file for nameservers to use
+**
+**  Parameters:
+**  	ar -- AR_LIB handle to update
+**  	file -- path to access
+**
+**  Return value:
+**  	0 -- success
+**  	!0 -- an error occurred; check errno
+**
+**  Notes:
+**  	Parse errors are not reported.
+**
+**  	The default set is not modified if no "nameserver" lines are found.
+*/
+
+int
+ar_resolvconf(AR_LIB ar, char *file)
+{
+	int af;
+	int n = 0;
+	FILE *f;
+	char *p;
+	char *sp;
+	SOCKADDR *news;
+	struct sockaddr_in s4;
+#ifdef AF_INET6
+	struct sockaddr_in6 s6;
+#endif /* AF_INET6 */
+	char buf[BUFRSZ];
+
+	assert(ar != NULL);
+	assert(file != NULL);
+
+	f = fopen(file, "r");
+	if (f == NULL)
+		return -1;
+
+	memset(buf, '\0', sizeof buf);
+
+	while (fgets(buf, sizeof buf - 1, f) != NULL)
+	{
+		sp = NULL;
+
+		for (p = buf; *p != '\0'; p++)
+		{
+			if (sp == NULL && isspace(*p))
+				sp = p;
+
+			if (*p == '#' || *p == '\n')
+			{
+				*p = '\0';
+				break;
+			}
+		}
+
+		if (sp != NULL)
+			*sp = '\0';
+
+		if (strcasecmp(buf, "nameserver") != 0)
+			continue;
+
+		af = -1;
+		if (inet_pton(AF_INET, sp + 1, &s4.sin_addr.s_addr) == 0)
+			af = AF_INET;
+#ifdef AF_INET6
+		else if (inet_pton(AF_INET6, sp + 1,
+		         &s6.sin6_addr.s6_addr) == 0)
+			af = AF_INET6;
+#endif /* AF_INET6 */
+
+		if (af == -1)
+			continue;
+
+		if (n == 0)
+		{
+			news = (SOCKADDR *) malloc(MAXNS * sizeof(SOCKADDR));
+			if (news == NULL)
+			{
+				fclose(f);
+				return -1;
+			}
+
+			free(ar->ar_nsaddrs);
+			ar->ar_nsaddrs = news;
+		}
+
+		if (af == AF_INET)
+		{
+			s4.sin_family = AF_INET;
+#ifdef HAVE_SIN_LEN
+			s4.sin_len = sizeof s4;
+#endif /* HAVE_SIN_LEN */
+			memcpy(&news[n], &s4, sizeof s4);
+		}
+#ifdef AF_INET6
+		else if (af == AF_INET6)
+		{
+			s6.sin6_family = AF_INET6;
+# ifdef HAVE_SIN6_LEN
+			s6.sin6_len = sizeof s6;
+# endif /* HAVE_SIN6_LEN */
+			memcpy(&news[n], &s6, sizeof s6);
+		}
+#endif /* AF_INET6 */
+
+		n++;
+		if (n == MAXNS)
+			break;
+	}
+
+	ar->ar_nscount = n;
+
+	fclose(f);
+
+	return 0;
+}
