@@ -2,7 +2,7 @@
 **  Copyright (c) 2007-2009 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 **
-**  Copyright (c) 2009, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2009, 2012, The OpenDKIM Project.  All rights reserved.
 */
 
 #ifndef lint
@@ -133,6 +133,7 @@ DKIM_STAT
 dkim_tmpfile(DKIM *dkim, int *fp, _Bool keep)
 {
 	int fd;
+	char *p;
 	char path[MAXPATHLEN + 1];
 
 	assert(dkim != NULL);
@@ -149,6 +150,14 @@ dkim_tmpfile(DKIM *dkim, int *fp, _Bool keep)
 		         dkim->dkim_libhandle->dkiml_tmpdir);
 	}
 
+	for (p = path + strlen(dkim->dkim_libhandle->dkiml_tmpdir) + 1;
+	     *p != '\0';
+	     p++)
+	{
+		if (*p == '/')
+			*p = '.';
+	}
+
 	fd = mkstemp(path);
 	if (fd == -1)
 	{
