@@ -2,19 +2,25 @@
 **  Copyright (c) 2005-2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 **
-**  Copyright (c) 2009, 2010, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2009-2011, The OpenDKIM Project.  All rights reserved.
 */
 
 #ifndef lint
 static char t_test134_c_id[] = "@(#)$Id: t-test134.c,v 1.2 2010/03/21 06:23:29 cm-msk Exp $";
 #endif /* !lint */
 
+#include "build-config.h"
+
 /* system includes */
 #include <sys/types.h>
 #include <assert.h>
 #include <string.h>
 #include <stdio.h>
 
+#ifdef USE_GNUTLS
+# include <gnutls/gnutls.h>
+#endif /* USE_GNUTLS */
+
 /* libopendkim includes */
 #include "../dkim.h"
 #include "t-testdata.h"
@@ -47,6 +53,10 @@ main(int argc, char **argv)
 
 	printf("*** relaxed/simple rsa-sha1 verifying with oversized domain name\n");
 
+#ifdef USE_GNUTLS
+	(void) gnutls_global_init();
+#endif /* USE_GNUTLS */
+
 	/* instantiate the library */
 	lib = dkim_init(NULL, NULL);
 	assert(lib != NULL);
