@@ -11,7 +11,7 @@
 #define _DKIM_CRYPTO_H_
 
 #ifndef lint
-static char dkim_crypto_h_id[] = "@(#)$Id: opendkim-crypto.h,v 1.2 2009/08/03 19:14:12 cm-msk Exp $";
+static char opendkim_crypto_h_id[] = "@(#)$Id: opendkim-crypto.h,v 1.2 2009/08/03 19:14:12 cm-msk Exp $";
 #endif /* !lint */
 
 #ifdef __STDC__
@@ -25,6 +25,9 @@ static char dkim_crypto_h_id[] = "@(#)$Id: opendkim-crypto.h,v 1.2 2009/08/03 19
 #endif /* __STDC__ */
 
 /* PROTOTYPES */
+#ifdef USE_GNUTLS
+extern const char *dkimf_crypto_geterror __P((void));
+#endif /* USE_GNUTLS */
 extern int dkimf_crypto_init __P((void));
 extern void dkimf_crypto_free __P((void));
 
