@@ -1,5 +1,5 @@
 /*
-**  Copyright (c) 2009-2011, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2009-2012, The OpenDKIM Project.  All rights reserved.
 **
 **  $Id: opendkim-lua.c,v 1.20 2010/09/14 18:23:38 cm-msk Exp $
 */
@@ -43,149 +43,155 @@ struct dkimf_lua_io
 	size_t		lua_io_alloc;
 };
 
+#if LUA_VERSION_NUM == 502
+# define ODKIM_PREFIX	"odkim."
+#else /* LUA_VERSION_NUM == 502 */
+# define ODKIM_PREFIX	""
+#endif /* LUA_VERSION_NUM == 502 */
+
 #ifdef DKIMF_LUA_CONTEXT_HOOKS
 /* libraries */
 static const luaL_Reg dkimf_lua_lib_setup[] =
 {
-	{ "check_popauth",	dkimf_xs_popauth	},
-	{ "db_check",		dkimf_xs_dbquery	},
-	{ "db_close",		dkimf_xs_dbclose	},
-	{ "db_open",		dkimf_xs_dbopen		},
+	{ ODKIM_PREFIX "check_popauth",		dkimf_xs_popauth	},
+	{ ODKIM_PREFIX "db_check",		dkimf_xs_dbquery	},
+	{ ODKIM_PREFIX "db_close",		dkimf_xs_dbclose	},
+	{ ODKIM_PREFIX "db_open",		dkimf_xs_dbopen		},
 # ifdef _FFR_LUA_GLOBALS
-	{ "export",		dkimf_xs_export		},
+	{ ODKIM_PREFIX "export",		dkimf_xs_export		},
 # endif /* _FFR_LUA_GLOBALS */
-	{ "get_clienthost",	dkimf_xs_clienthost	},
-	{ "get_clientip",	dkimf_xs_clientip	},
-	{ "get_dbhandle",	dkimf_xs_dbhandle	},
-	{ "get_fromdomain",	dkimf_xs_fromdomain	},
-	{ "get_header",		dkimf_xs_getheader	},
-	{ "get_mtasymbol",	dkimf_xs_getsymval	},
-	{ "get_rcpt",		dkimf_xs_rcpt		},
-	{ "get_rcptarray",	dkimf_xs_rcptarray	},
-	{ "internal_ip",	dkimf_xs_internalip	},
-	{ "log",		dkimf_xs_log		},
-	{ "rcpt_count",		dkimf_xs_rcptcount	},
-	{ "replace_header",	dkimf_xs_replaceheader	},
-	{ "resign",		dkimf_xs_resign		},
-	{ "set_result",		dkimf_xs_setresult	},
-	{ "sign",		dkimf_xs_requestsig	},
+	{ ODKIM_PREFIX "get_clienthost",	dkimf_xs_clienthost	},
+	{ ODKIM_PREFIX "get_clientip",		dkimf_xs_clientip	},
+	{ ODKIM_PREFIX "get_dbhandle",		dkimf_xs_dbhandle	},
+	{ ODKIM_PREFIX "get_envfrom",		dkimf_xs_getenvfrom	},
+	{ ODKIM_PREFIX "get_fromdomain",	dkimf_xs_fromdomain	},
+	{ ODKIM_PREFIX "get_header",		dkimf_xs_getheader	},
+	{ ODKIM_PREFIX "get_mtasymbol",		dkimf_xs_getsymval	},
+	{ ODKIM_PREFIX "get_rcpt",		dkimf_xs_rcpt		},
+	{ ODKIM_PREFIX "get_rcptarray",		dkimf_xs_rcptarray	},
+	{ ODKIM_PREFIX "internal_ip",		dkimf_xs_internalip	},
+	{ ODKIM_PREFIX "log",			dkimf_xs_log		},
+	{ ODKIM_PREFIX "rcpt_count",		dkimf_xs_rcptcount	},
+	{ ODKIM_PREFIX "replace_header",	dkimf_xs_replaceheader	},
+	{ ODKIM_PREFIX "resign",		dkimf_xs_resign		},
+	{ ODKIM_PREFIX "set_result",		dkimf_xs_setresult	},
+	{ ODKIM_PREFIX "sign",			dkimf_xs_requestsig	},
 #ifdef _FFR_REPUTATION
-	{ "spam",		dkimf_xs_spam		},
+	{ ODKIM_PREFIX "spam",			dkimf_xs_spam		},
 #endif /* _FFR_REPUTATION */
-	{ "use_ltag",		dkimf_xs_setpartial	},
-	{ "verify",		dkimf_xs_verify		},
-# ifdef _FFR_XTAGS
-	{ "xtag",		dkimf_xs_xtag		},
-# endif /* _FFR_XTAGS */
-	{ NULL,			NULL			}
+	{ ODKIM_PREFIX "use_ltag",		dkimf_xs_setpartial	},
+	{ ODKIM_PREFIX "verify",		dkimf_xs_verify		},
+	{ ODKIM_PREFIX "xtag",			dkimf_xs_xtag		},
+	{ NULL,					NULL			}
 };
 
 static const luaL_Reg dkimf_lua_lib_screen[] =
 {
-	{ "db_check",		dkimf_xs_dbquery	},
-	{ "db_close",		dkimf_xs_dbclose	},
-	{ "db_open",		dkimf_xs_dbopen		},
+	{ ODKIM_PREFIX "db_check",		dkimf_xs_dbquery	},
+	{ ODKIM_PREFIX "db_close",		dkimf_xs_dbclose	},
+	{ ODKIM_PREFIX "db_open",		dkimf_xs_dbopen		},
 # ifdef _FFR_LUA_GLOBALS
-	{ "export",		dkimf_xs_export		},
+	{ ODKIM_PREFIX "export",		dkimf_xs_export		},
 # endif /* _FFR_LUA_GLOBALS */
-	{ "get_dbhandle",	dkimf_xs_dbhandle	},
-	{ "get_fromdomain",	dkimf_xs_fromdomain	},
-	{ "get_header",		dkimf_xs_getheader	},
-	{ "get_mtasymbol",	dkimf_xs_getsymval	},
-	{ "get_rcpt",		dkimf_xs_rcpt		},
-	{ "get_rcptarray",	dkimf_xs_rcptarray	},
-	{ "get_sigarray",	dkimf_xs_getsigarray	},
-	{ "get_sigcount",	dkimf_xs_getsigcount	},
-	{ "get_sighandle",	dkimf_xs_getsighandle	},
-	{ "log",		dkimf_xs_log		},
-	{ "parse_field",	dkimf_xs_parsefield	},
-	{ "rcpt_count",		dkimf_xs_rcptcount	},
-	{ "sig_getdomain",	dkimf_xs_getsigdomain	},
-	{ "sig_getidentity",	dkimf_xs_getsigidentity	},
-	{ "sig_ignore",		dkimf_xs_sigignore	},
+	{ ODKIM_PREFIX "get_dbhandle",		dkimf_xs_dbhandle	},
+	{ ODKIM_PREFIX "get_envfrom",		dkimf_xs_getenvfrom	},
+	{ ODKIM_PREFIX "get_fromdomain",	dkimf_xs_fromdomain	},
+	{ ODKIM_PREFIX "get_header",		dkimf_xs_getheader	},
+	{ ODKIM_PREFIX "get_mtasymbol",		dkimf_xs_getsymval	},
+	{ ODKIM_PREFIX "get_rcpt",		dkimf_xs_rcpt		},
+	{ ODKIM_PREFIX "get_rcptarray",		dkimf_xs_rcptarray	},
+	{ ODKIM_PREFIX "get_sigarray",		dkimf_xs_getsigarray	},
+	{ ODKIM_PREFIX "get_sigcount",		dkimf_xs_getsigcount	},
+	{ ODKIM_PREFIX "get_sighandle",		dkimf_xs_getsighandle	},
+	{ ODKIM_PREFIX "log",			dkimf_xs_log		},
+	{ ODKIM_PREFIX "parse_field",		dkimf_xs_parsefield	},
+	{ ODKIM_PREFIX "rcpt_count",		dkimf_xs_rcptcount	},
+	{ ODKIM_PREFIX "sig_getdomain",		dkimf_xs_getsigdomain	},
+	{ ODKIM_PREFIX "sig_getidentity",	dkimf_xs_getsigidentity	},
+	{ ODKIM_PREFIX "sig_ignore",		dkimf_xs_sigignore	},
 #ifdef _FFR_REPUTATION
-	{ "spam",		dkimf_xs_spam		},
+	{ ODKIM_PREFIX "spam",			dkimf_xs_spam		},
 #endif /* _FFR_REPUTATION */
-	{ NULL,			NULL			}
+	{ NULL,					NULL			}
 };
 
 # ifdef _FFR_STATSEXT
 static const luaL_Reg dkimf_lua_lib_stats[] =
 {
 #  ifdef _FFR_LUA_GLOBALS
-	{ "export",		dkimf_xs_export		},
+	{ ODKIM_PREFIX "export",		dkimf_xs_export		},
 #  endif /* _FFR_LUA_GLOBALS */
-	{ "get_header",		dkimf_xs_getheader	},
-	{ "get_mtasymbol",	dkimf_xs_getsymval	},
-	{ "get_policy",		dkimf_xs_getpolicy	},
-	{ "get_rcpt",		dkimf_xs_rcpt		},
-	{ "get_rcptarray",	dkimf_xs_rcptarray	},
-	{ "get_reputation",	dkimf_xs_getreputation	},
-	{ "get_sigarray",	dkimf_xs_getsigarray	},
-	{ "get_sigcount",	dkimf_xs_getsigcount	},
-	{ "get_sighandle",	dkimf_xs_getsighandle	},
-	{ "log",		dkimf_xs_log		},
-	{ "parse_field",	dkimf_xs_parsefield	},
+	{ ODKIM_PREFIX "get_envfrom",		dkimf_xs_getenvfrom	},
+	{ ODKIM_PREFIX "get_header",		dkimf_xs_getheader	},
+	{ ODKIM_PREFIX "get_mtasymbol",		dkimf_xs_getsymval	},
+	{ ODKIM_PREFIX "get_policy",		dkimf_xs_getpolicy	},
+	{ ODKIM_PREFIX "get_rcpt",		dkimf_xs_rcpt		},
+	{ ODKIM_PREFIX "get_rcptarray",		dkimf_xs_rcptarray	},
+	{ ODKIM_PREFIX "get_reputation",	dkimf_xs_getreputation	},
+	{ ODKIM_PREFIX "get_sigarray",		dkimf_xs_getsigarray	},
+	{ ODKIM_PREFIX "get_sigcount",		dkimf_xs_getsigcount	},
+	{ ODKIM_PREFIX "get_sighandle",		dkimf_xs_getsighandle	},
+	{ ODKIM_PREFIX "log",			dkimf_xs_log		},
+	{ ODKIM_PREFIX "parse_field",		dkimf_xs_parsefield	},
 #  ifdef _FFR_LUA_GLOBALS
 #   ifdef _FFR_RBL
-	{ "rbl_check",		dkimf_xs_rblcheck	},
+	{ ODKIM_PREFIX "rbl_check",		dkimf_xs_rblcheck	},
 #   endif /* _FFR_RBL */
 #  endif /* _FFR_LUA_GLOBALS */
-	{ "rcpt_count",		dkimf_xs_rcptcount	},
-	{ "sig_bhresult",	dkimf_xs_sigbhresult	},
-	{ "sig_bodylength",	dkimf_xs_bodylength	},
-	{ "sig_canonlength",	dkimf_xs_canonlength	},
-	{ "sig_getdomain",	dkimf_xs_getsigdomain	},
-	{ "sig_getidentity",	dkimf_xs_getsigidentity	},
-	{ "sig_result",		dkimf_xs_sigresult	},
-	{ "stats",		dkimf_xs_statsext	},
+	{ ODKIM_PREFIX "rcpt_count",		dkimf_xs_rcptcount	},
+	{ ODKIM_PREFIX "sig_bhresult",		dkimf_xs_sigbhresult	},
+	{ ODKIM_PREFIX "sig_bodylength",	dkimf_xs_bodylength	},
+	{ ODKIM_PREFIX "sig_canonlength",	dkimf_xs_canonlength	},
+	{ ODKIM_PREFIX "sig_getdomain",		dkimf_xs_getsigdomain	},
+	{ ODKIM_PREFIX "sig_getidentity",	dkimf_xs_getsigidentity	},
+	{ ODKIM_PREFIX "sig_result",		dkimf_xs_sigresult	},
+	{ ODKIM_PREFIX "stats",			dkimf_xs_statsext	},
 #ifdef _FFR_REPUTATION
-	{ "spam",		dkimf_xs_spam		},
+	{ ODKIM_PREFIX "spam",			dkimf_xs_spam		},
 #endif /* _FFR_REPUTATION */
-	{ NULL,			NULL			}
+	{ NULL,					NULL			}
 };
 # endif /* _FFR_STATSEXT */
 
 static const luaL_Reg dkimf_lua_lib_final[] =
 {
-	{ "add_header",		dkimf_xs_addheader	},
-	{ "add_rcpt",		dkimf_xs_addrcpt	},
-	{ "del_header",		dkimf_xs_delheader	},
-	{ "del_rcpt",		dkimf_xs_delrcpt	},
-	{ "get_clienthost",	dkimf_xs_clienthost	},
-	{ "get_clientip",	dkimf_xs_clientip	},
-	{ "get_fromdomain",	dkimf_xs_fromdomain	},
-	{ "get_header",		dkimf_xs_getheader	},
-	{ "get_mtasymbol",	dkimf_xs_getsymval	},
-	{ "get_policy",		dkimf_xs_getpolicy	},
-	{ "get_rcpt",		dkimf_xs_rcpt		},
-	{ "get_rcptarray",	dkimf_xs_rcptarray	},
-	{ "get_reputation",	dkimf_xs_getreputation	},
-	{ "get_sigarray",	dkimf_xs_getsigarray	},
-	{ "get_sigcount",	dkimf_xs_getsigcount	},
-	{ "get_sighandle",	dkimf_xs_getsighandle	},
-	{ "log",		dkimf_xs_log		},
-	{ "quarantine",		dkimf_xs_quarantine	},
-	{ "parse_field",	dkimf_xs_parsefield	},
+	{ ODKIM_PREFIX "add_header",		dkimf_xs_addheader	},
+	{ ODKIM_PREFIX "add_rcpt",		dkimf_xs_addrcpt	},
+	{ ODKIM_PREFIX "del_header",		dkimf_xs_delheader	},
+	{ ODKIM_PREFIX "del_rcpt",		dkimf_xs_delrcpt	},
+	{ ODKIM_PREFIX "get_clienthost",	dkimf_xs_clienthost	},
+	{ ODKIM_PREFIX "get_clientip",		dkimf_xs_clientip	},
+	{ ODKIM_PREFIX "get_envfrom",		dkimf_xs_getenvfrom	},
+	{ ODKIM_PREFIX "get_fromdomain",	dkimf_xs_fromdomain	},
+	{ ODKIM_PREFIX "get_header",		dkimf_xs_getheader	},
+	{ ODKIM_PREFIX "get_mtasymbol",		dkimf_xs_getsymval	},
+	{ ODKIM_PREFIX "get_policy",		dkimf_xs_getpolicy	},
+	{ ODKIM_PREFIX "get_rcpt",		dkimf_xs_rcpt		},
+	{ ODKIM_PREFIX "get_rcptarray",		dkimf_xs_rcptarray	},
+	{ ODKIM_PREFIX "get_reputation",	dkimf_xs_getreputation	},
+	{ ODKIM_PREFIX "get_sigarray",		dkimf_xs_getsigarray	},
+	{ ODKIM_PREFIX "get_sigcount",		dkimf_xs_getsigcount	},
+	{ ODKIM_PREFIX "get_sighandle",		dkimf_xs_getsighandle	},
+	{ ODKIM_PREFIX "log",			dkimf_xs_log		},
+	{ ODKIM_PREFIX "quarantine",		dkimf_xs_quarantine	},
+	{ ODKIM_PREFIX "parse_field",		dkimf_xs_parsefield	},
 # ifdef _FFR_RBL
-	{ "rbl_check",		dkimf_xs_rblcheck	},
+	{ ODKIM_PREFIX "rbl_check",		dkimf_xs_rblcheck	},
 # endif /* _FFR_RBL */
-	{ "rcpt_count",		dkimf_xs_rcptcount	},
-	{ "set_reply",		dkimf_xs_setreply	},
-	{ "set_result",		dkimf_xs_setresult	},
-	{ "sig_bhresult",	dkimf_xs_sigbhresult	},
-	{ "sig_bodylength",	dkimf_xs_bodylength	},
-	{ "sig_canonlength",	dkimf_xs_canonlength	},
-	{ "sig_getdomain",	dkimf_xs_getsigdomain	},
-	{ "sig_getidentity",	dkimf_xs_getsigidentity	},
-	{ "sig_result",		dkimf_xs_sigresult	},
+	{ ODKIM_PREFIX "rcpt_count",		dkimf_xs_rcptcount	},
+	{ ODKIM_PREFIX "set_reply",		dkimf_xs_setreply	},
+	{ ODKIM_PREFIX "set_result",		dkimf_xs_setresult	},
+	{ ODKIM_PREFIX "sig_bhresult",		dkimf_xs_sigbhresult	},
+	{ ODKIM_PREFIX "sig_bodylength",	dkimf_xs_bodylength	},
+	{ ODKIM_PREFIX "sig_canonlength",	dkimf_xs_canonlength	},
+	{ ODKIM_PREFIX "sig_getdomain",		dkimf_xs_getsigdomain	},
+	{ ODKIM_PREFIX "sig_getidentity",	dkimf_xs_getsigidentity	},
+	{ ODKIM_PREFIX "sig_result",		dkimf_xs_sigresult	},
 # ifdef _FFR_REPUTATION
-	{ "spam",		dkimf_xs_spam		},
+	{ ODKIM_PREFIX "spam",			dkimf_xs_spam		},
 # endif /* _FFR_REPUTATION */
-# ifdef _FFR_XTAGS
-	{ "xtag",		dkimf_xs_xtag		},
-# endif /* _FFR_XTAGS */
-	{ NULL,			NULL			}
+	{ ODKIM_PREFIX "xtag",			dkimf_xs_xtag		},
+	{ NULL,					NULL			}
 };
 #endif /* DKIMF_LUA_CONTEXT_HOOKS */
 
@@ -299,7 +305,11 @@ dkimf_lua_alloc(void *ud, void *ptr, size_t osize, size_t nsize)
 		free(ptr);
 		return NULL;
 	}
+# if LUA_VERSION_NUM == 502
+	else if (nsize != 0 && ptr == NULL)
+# else /* LUA_VERSION_NUM == 502 */
 	else if (nsize != 0 && osize == 0)
+# endif /* LUA_VERSION_NUM == 502 */
 	{
 		return malloc(nsize);
 	}
@@ -495,7 +505,11 @@ dkimf_lua_setup_hook(void *ctx, const char *script, size_t scriptlen,
 	**  Register functions.
 	*/
 
+# if LUA_VERSION_NUM == 502
+	luaL_setfunc(l, dkimf_lua_lib_setup, 0);
+# else /* LUA_VERSION_NUM == 502 */
 	luaL_register(l, "odkim", dkimf_lua_lib_setup);
+# endif /* LUA_VERSION_NUM == 502 */
 	lua_pop(l, 1);
 
 	/*
@@ -537,7 +551,11 @@ dkimf_lua_setup_hook(void *ctx, const char *script, size_t scriptlen,
 	dkimf_import_globals(ctx, l);
 # endif /* _FFR_LUA_GLOBALS */
 
+# if LUA_VERSION_NUM == 502
+	switch (lua_load(l, dkimf_lua_reader, (void *) &io, name, NULL))
+# else /* LUA_VERSION_NUM == 502 */
 	switch (lua_load(l, dkimf_lua_reader, (void *) &io, name))
+# endif /* LUA_VERSION_NUM == 502 */
 	{
 	  case 0:
 		break;
@@ -641,7 +659,11 @@ dkimf_lua_screen_hook(void *ctx, const char *script, size_t scriptlen,
 	**  Register functions.
 	*/
 
+# if LUA_VERSION_NUM == 502
+	luaL_setfuncs(l, dkimf_lua_lib_screen, 0);
+# else /* LUA_VERSION_NUM == 502 */
 	luaL_register(l, "odkim", dkimf_lua_lib_screen);
+# endif /* LUA_VERSION_NUM == 502 */
 	lua_pop(l, 1);
 
 	/*
@@ -673,7 +695,11 @@ dkimf_lua_screen_hook(void *ctx, const char *script, size_t scriptlen,
 	dkimf_import_globals(ctx, l);
 # endif /* _FFR_LUA_GLOBALS */
 
+# if LUA_VERSION_NUM == 502
+	switch (lua_load(l, dkimf_lua_reader, (void *) &io, name, NULL))
+# else /* LUA_VERSION_NUM == 502 */
 	switch (lua_load(l, dkimf_lua_reader, (void *) &io, name))
+# endif /* LUA_VERSION_NUM == 502 */
 	{
 	  case 0:
 		break;
@@ -777,7 +803,11 @@ dkimf_lua_stats_hook(void *ctx, const char *script, size_t scriptlen,
 	**  Register functions.
 	*/
 
+# if LUA_VERSION_NUM == 502
+	luaL_setfuncs(l, dkimf_lua_lib_stats, 0);
+# else /* LUA_VERSION_NUM == 502 */
 	luaL_register(l, "odkim", dkimf_lua_lib_stats);
+# endif /* LUA_VERSION_NUM == 502 */
 	lua_pop(l, 1);
 
 	/*
@@ -915,7 +945,11 @@ dkimf_lua_stats_hook(void *ctx, const char *script, size_t scriptlen,
 	dkimf_import_globals(ctx, l);
 # endif /* _FFR_LUA_GLOBALS */
 
+# if LUA_VERSION_NUM == 502
+	switch (lua_load(l, dkimf_lua_reader, (void *) &io, name, NULL))
+# else /* LUA_VERSION_NUM == 502 */
 	switch (lua_load(l, dkimf_lua_reader, (void *) &io, name))
+# endif /* LUA_VERSION_NUM == 502 */
 	{
 	  case 0:
 		break;
@@ -1019,7 +1053,11 @@ dkimf_lua_final_hook(void *ctx, const char *script, size_t scriptlen,
 	**  Register functions.
 	*/
 
+# if LUA_VERSION_NUM == 502
+	luaL_setfuncs(l, dkimf_lua_lib_final, 0);
+# else /* LUA_VERSION_NUM == 502 */
 	luaL_register(l, "odkim", dkimf_lua_lib_final);
+# endif /* LUA_VERSION_NUM == 502 */
 	lua_pop(l, 1);
 
 	/*
@@ -1157,7 +1195,11 @@ dkimf_lua_final_hook(void *ctx, const char *script, size_t scriptlen,
 	dkimf_import_globals(ctx, l);
 # endif /* _FFR_LUA_GLOBALS */
 
+# if LUA_VERSION_NUM == 502
+	switch (lua_load(l, dkimf_lua_reader, (void *) &io, name, NULL))
+# else /* LUA_VERSION_NUM == 502 */
 	switch (lua_load(l, dkimf_lua_reader, (void *) &io, name))
+# endif /* LUA_VERSION_NUM == 502 */
 	{
 	  case 0:
 		break;
