@@ -1,7 +1,7 @@
 <?php
 
 ###
-### Copyright (c) 2011, The OpenDKIM Project.  All rights reserved.
+### Copyright (c) 2012, The OpenDKIM Project.  All rights reserved.
 ###
 
 # Database configuration for REPUTE queries
@@ -18,4 +18,7 @@ $repute_pwd = "opendkim";
 # database name
 $repute_dbname = "opendkim";
 
+# use JSON?  if yes, set this to 1
+$use_json = 0;
+
 ?>
